@@ -1,6 +1,7 @@
 //! Contains traits defining required properties of the underlying integer types.
 
-use core::fmt::Display;
+use core::fmt;
+use core::iter;
 use core::ops;
 
 /// Required arithmetic operations for integers.
@@ -15,6 +16,13 @@ where
     Self: ops::Mul<Output = Self> + for<'a> ops::Mul<&'a Self, Output = Self>,
     Self: ops::Div<Output = Self> + for<'a> ops::Div<&'a Self, Output = Self>,
     Self: ops::Neg<Output = Self>,
+    Self: ops::AddAssign<Self> + for<'a> ops::AddAssign<&'a Self>,
+    Self: ops::SubAssign<Self> + for<'a> ops::SubAssign<&'a Self>,
+    Self: ops::MulAssign<Self> + for<'a> ops::MulAssign<&'a Self>,
+    Self: ops::DivAssign<Self> + for<'a> ops::DivAssign<&'a Self>,
+    Self: ops::RemAssign<Self> + for<'a> ops::RemAssign<&'a Self>,
+    Self: iter::Sum<Self> + for<'a> iter::Sum<&'a Self>,
+    Self: iter::Product<Self> + for<'a> iter::Product<&'a Self>,
 {
 }
 
@@ -22,20 +30,19 @@ where
 pub trait Integer
 where
     Self: Copy,
-    Self: Display,
-    Self: From<i32>,
+    Self: fmt::Display,
+    Self: TryFrom<usize>,
     Self: Ord,
     Self: IntegerOps,
 {
     /// Returns zero.
-    fn zero() -> Self {
-        Self::from(0)
-    }
+    fn zero() -> Self;
 
     /// Returns one.
-    fn one() -> Self {
-        Self::from(1)
-    }
+    fn one() -> Self;
+
+    /// Returns two.
+    fn two() -> Self;
 
     /// The absolute value function.
     fn abs(self) -> Self;
@@ -44,6 +51,23 @@ where
     ///
     /// Returns `1` for positive numbers, `0` for zero and `-1` for negative numbers.
     fn signum(self) -> Self;
+
+    /// Converts to an `usize` if possible, or panics otherwise.
+    fn to_usize(self) -> usize
+    where
+        usize: TryFrom<Self>,
+        <usize as TryFrom<Self>>::Error: fmt::Debug,
+    {
+        usize::try_from(self).unwrap()
+    }
+
+    /// Converts from an `usize` if possible, or panics otherwise.
+    fn from_usize(n: usize) -> Self
+    where
+        <Self as TryFrom<usize>>::Error: fmt::Debug,
+    {
+        Self::try_from(n).unwrap()
+    }
 }
 
 #[doc(hidden)]
@@ -52,6 +76,15 @@ macro_rules! impl_integer {
         impl IntegerOps for $t {}
 
         impl Integer for $t {
+            fn zero() -> Self {
+                0
+            }
+            fn one() -> Self {
+                1
+            }
+            fn two() -> Self {
+                2
+            }
             fn abs(self) -> Self {
                 <$t>::abs(self)
             }
@@ -62,9 +95,12 @@ macro_rules! impl_integer {
     };
 }
 
+impl_integer!(i8);
+impl_integer!(i16);
 impl_integer!(i32);
 impl_integer!(i64);
 impl_integer!(i128);
+impl_integer!(isize);
 
 #[cfg(test)]
 mod tests {
