@@ -266,6 +266,23 @@ impl<S: Integer> iter::FromIterator<S> for Vec3d<S> {
     }
 }
 
+impl<S: Integer> iter::Sum<Vec3d<S>> for Vec3d<S> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = Vec3d<S>>,
+    {
+        iter.fold(Vec3d::zero(), |sum, v| sum + v)
+    }
+}
+impl<'a, S: Integer> iter::Sum<&'a Vec3d<S>> for Vec3d<S> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = &'a Vec3d<S>>,
+    {
+        iter.fold(Vec3d::zero(), |sum, v| sum + v)
+    }
+}
+
 impl<S: Integer> fmt::Debug for Vec3d<S> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "(")?;
@@ -587,6 +604,14 @@ mod tests {
         assert_eq!(v3d(1, 2, 3), vec![1, 2, 3].into_iter().collect::<Vec3d>());
     }
 
+    #[test]
+    #[test]
+    fn test_sum() {
+        let vs = vec![v3d(1, 0, -1), v3d(0, 1, 2), v3d(2, -3, 1)];
+        assert_eq!(v3d(3, -2, 2), vs.iter().map(|v: &Vec3d| v).sum());
+        assert_eq!(v3d(3, -2, 2), vs.into_iter().map(|v: Vec3d| v).sum());
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("(2, -3, 5)", format!("{:?}", v3d(2, -3, 5)));
